// deHarry, 2024-02-01
// Programmentwicklung und Debugging erfolgte auf einem Wemos D1 mini. 
//	Letzlich wird ein ESP-01 die erfassten und aufsummierten Daten im WLAN zur Verfügung stellen. 
//
// Hardware:
// Adapter ist entweder für die Wasseruhr ein induktiver Näherungssensor LJ12-A3-4-Z/BX in einem 3D gedruckten Halter...
// (Projekt siehe https://harald-sattler.de/html/adapter_wasseruhr.htm)
// Die Anschlüse sind etwas ungewöhnlich gefärbt:
//	Braun: Plus 6..36 V
//	Blau: GND
//	Schwarz: Signal (vorher messen, was da rauskommt!! (OC oder Spannung)
// Test: Der Sensor funktioniert auch mit 5 V, die Versorgung des ESP01 kann also mittels 
//	USB Netzteil und StepDown Wandler erfolgen.
//
// ...oder für den Gaszähler ein Halleffektensor TLE4913 auf Breakout Board in einem 3D gedruckten Halter.
//	Hier muss die passende Stelle im Ausschnitt des Gaszählergehäuses gefunden werden. Kann etwas spannend werden, 
//	denn der Zähler gibt seine Impulse auf der zweitletzten Stelle aus. Da muss also einiges Gas durchrauschen,
//	bis der nächste Impuls kommt. (Projekt siehe https://harald-sattler.de/html/adapter_gaszahler.htm)
//	Hinweis: Der ursprünglich vorgesehene A3144 ist für den Magneten im Gaszähler zu unempfindlich.
//
//  Achtung! Pullup muss beim ESP-01 10 kΩ sein (ist im Layout vorgesehen) oder INPUT_PULLUP verwenden.
//
// Am ESP ist ein OLED Display per I2C angeflanscht das per Knopfdruck aktiviert werden kann (normalerweise AUS um die Pixel zu schonen).
//	Der Code erkennt automatisch den verwendeten Typ Microprozessor und passt die Ausgänge an.
//
// Software:
// Einlesen der Impulse des Näherungssensors bei Wasser, 1 Imp/Liter
//	oder
// Einlesen der Impulse des Halleffektsensors bei Gas, 1 Imp/0,01 m³
//	und jeweils die Summe bilden und ausgeben auf dem OLED Display und über ein WEB Interface (Anzeige fürs Auge und Abfragen
//	der Werte per Aufrufschnittstelle).
//
// Der WiFi Teil des Code wurde auf einem Wemos D1 mini lauffähig gemacht. Der Upload auf den ESP läuft beim Wemos doppelt so schnell 
//	wie beim ESP-01 und man muss ihn nicht zum Programmieren erst auf einen Programmer umstecken. 
//  Außerdem gibt es Debugausgaben auf der seriellen SS.
// Update des Programms kann OTA erfolgen, die Routinen sind eingebaut.
//  Das geht auch auf dem ESP-01 vergleichsweise flott, aber der Neuanlauf dauert etwas (~18 s)... nicht ungeduldig werden ;)
//
// Anleihe bei FipsOK (https://fipsok.de/Projekt/gaszaehler-esp8266-nodemcu)
//	- Zählerstand bei Änderung alle 4 Stunden im Flash speichern
//	- HTML Seite mit Anzeige und Möglichkeit den aktuellen Zählerstand zu setzen
//
// Version V1.1, 09.04.2024
//	- Bei langem Druck (> 2 s) auf den Button wird der momentane Zählerstand ins Flash geschrieben, mit Rückmeldung auf OLED
// Version V1.2, 10.04.2024
//	- Ausgabe im WEB Interface mit konstant 5 Vorkommastellen (Zero padded), so dass der Dezimalpunkt immer auf der roten Linie sitzt
//

// WiFi libs
#include <WiFiServer.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <ArduinoOTA.h>					// https://arduino-esp8266.readthedocs.io/en/latest/ota_updates/readme.html

// LittleFS
#include <LittleFS.h>                   // http://arduino-esp8266.readthedocs.io/en/latest/filesystem.html

// Bibliothek für das OLED Display
#include <U8g2lib.h>

// Sonstige Libs

// Defines für Debug ----------------------------------------------------------------------------------
#if defined (ARDUINO_ESP8266_GENERIC)	// ESP-01
	#undef SERIALDEBUG					// beim ESP-01 darf Serial nicht aktiviert werden
#else
	#define SERIALDEBUG					// jede Menge Serial Ausgaben für Debugzwecke aktivieren
#endif
#ifdef SERIALDEBUG						// wenn aktiviert, Ausgaben auf Serial
	#define debugPrint Serial.print
	#define debugPrintf Serial.printf
	#define debugPrintln Serial.println
#else									// keine Ausgaben auf Serial
	#define debugPrint		//Serial.print
	#define debugPrintf		//Serial.printf
	#define debugPrintln	//Serial.println
#endif
// ----------------------------------------------------------------------------------------------------

// allgemeine Defines
//#define WASSER							// Code und Anzeige für Wasserzähler sonst für Gaszähler
#define USEIRQ							// Sensorabfrage per Interrrupt - muss immer aktiv sein
#define USEFS							// Zählerwert im LittleFS ablegen

#if defined WASSER						// mit Blanks zum "zentrieren" auf dem OLED Display
	#define COUNTERNAME "   Sensus 420"
#else
	#define COUNTERNAME "elster BK-G4M"
#endif

// Credentials für das eigene WLAN
#define OwnSSID "OwnSSID"				// -->> hier die eigenen Credentials eintragen <<--
#define OwnPassword "OwnPassword"		// -->> hier die eigenen Credentials eintragen <<--

// Pin Definitionen
#if defined (ARDUINO_ESP8266_WEMOS_D1MINI)	// Wemos D1 R2
	// I2C Pins am Wemos D1 R2
	#define I2CSDA D1						// beim ESP01 1, beim Wemos 5
	#define I2CSCL D2						// beim ESP01 2, beim Wemos 4
	#define SENSORPIN D3					// beim ESP01 3, beim Wemos 3
	#define TASTER D0						// beim ESP01 0, beim Wemos 0	Achtung! D0 (= GPIO16) generiert keinen IRQ
	#define SIGNALLED D4					// beim ESP01 4, beim Wemos 2
#elif defined (ARDUINO_ESP8266_GENERIC)		// ESP-01
	// I2C Pins am ESP-01
	#define I2CSDA 1						// beim ESP01 1, beim Wemos 5
	#define I2CSCL 2						// beim ESP01 2, beim Wemos 4
	#define SENSORPIN 3						// beim ESP01 3, beim Wemos 3
	#define TASTER 0						// beim ESP01 0, beim Wemos 0	Achtung! D0 (= GPIO16) generiert keinen IRQ
#endif

// OLED Display instantiieren
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define SCREEN_ADDRESS 0x3C

U8G2_SSD1306_128X64_NONAME_F_HW_I2C myDisplay(U8G2_R0, U8X8_PIN_NONE, I2CSCL, I2CSDA);

// globale Variablen
unsigned long DisplayTimer = 0;						// für Timeout des Display, Pixel schonen durch Abschalten
bool DisplayContentEin = true;						// Ausgaben auf dem Display de-/aktivieren (PowerSave funktioniert nicht)
#define DISPLAYPERIOD 25000							// OLED Display nach TASTER==LOW für 25 s einschalten
bool clearOnce = true;								// Display einmalig nach Setup löschen

unsigned long counterValue = 8859060;				// Gaszähler als uLong, muss zur Anzeige durch 1000 geteilt werden
unsigned long oldCounterValue;						// Zählerwert im EEPROM speichern

float value, oldvalue;								// Zählerwerte für Fips' Teil des Code

// WiFiServer instantiieren
ESP8266WebServer server(80);

unsigned long clientDelayTimer = 0;
#define CLIENTDELAY 2000							// Werte alle 2 s abfragen und WEB Oberfläche aktualisieren

// globale Variablen
char buffer[20];									// Puffer für formatierte Ausgabe der Zahlen
uint8_t YOFF = 30;									// initialer vertikaler Offset für die Auflistung der Initialisierungen

unsigned long letzterAufruf = 0;					// Merker für "alle 4 h Zählerwert speichern"

unsigned long activityTimer = 0;					// Aktivitätsanzeige - Programm läuft noch ;)
uint8_t propellerState = 0;
//char propellerString[] = { '|', '/', '-', '\\' };	// langsam drehender Propeller auf dem OLED Display
char propellerString[] = { 'T', '+', 'T', '+' };	// weit genug nach rechts blinkt nur ein Pixel (linke Ecke des T)

// Taster
bool bTasterTrig = false;							// Tastendruckdauer auswerten
bool bTasterLang = false;							// Tastendruckdauer auswerten
unsigned long uTasterLang = 0;
#define LONGKEY 2000								// Zeitdauer für langen Tastendruck in ms
bool bsaved = false;


// Abfrage des Sensors per Interrupt ----------------------------------
#if defined USEIRQ
struct Sensor {
	const uint8_t PIN;
	bool triggered;
};

volatile Sensor SensorTLE = { SENSORPIN, false };	// muss volatile deklariert werden, sonst wird "triggered" nicht aktualisiert

//variables to keep track of the timing of recent interrupts
unsigned long sensor_time = 0;
unsigned long last_sensor_time = 0;

void ICACHE_RAM_ATTR sensorISR() {
	sensor_time = millis();
	if (sensor_time - last_sensor_time > 500)		// Prellen unterdrücken 
	{												// Tritt bei den verwendeten Sensoren nicht auf, schadet aber auch nicht :)
		SensorTLE.triggered = true;
		last_sensor_time = sensor_time;
	}
}
#endif
// \Abfrage des Sensors per Interrupt ----------------------------------


// ---------------------------------------------------------------------------------------------------------------------
void setup() {
#if defined USEIRQ
	pinMode(SensorTLE.PIN, INPUT_PULLUP);
	attachInterrupt(SensorTLE.PIN, sensorISR, FALLING);	// Sensor/Taster ziehen den Eingang auf Null/GND
#else
	pinMode(SENSORPIN, INPUT_PULLUP);				// Eingang für Sensor (Halleffektsensor TLE4913 oder Näherungssensor LJ12-A3-4-Z/BX)
#endif
	pinMode(TASTER, INPUT_PULLUP);					// Taster für Display temporär einschalten

#if defined (ARDUINO_ESP8266_WEMOS_D1MINI)			// LED nur beim Wemos vorhanden
	pinMode(SIGNALLED, OUTPUT);						// signal incoming data on serial
	digitalWrite(SIGNALLED, LOW);
#endif

// ACHTUNG - Wenn Serial.begin() aufgerufen wird, wird der RX-Pin des ESP-01 bedingungslos auf die serielle SS geroutet,
//				es ist dann nicht möglich, den Pin als normalen IO zu verwenden.
//			Siehe "https://www.forward.com.au/pfod/ESP8266/GPIOpins/ESP8266_01_pin_magic.html"

#ifdef SERIALDEBUG									// wenn aktiviert, Ausgaben auf Serial
	// bring up serial port
	Serial.begin(115200);							// Default ist Baudrate + 8N1
#endif

	// init OLED myDisplay, I2C-Adr. 0x3c
	bool result = myDisplay.begin();

#define LUCIDA										// welcher Font. Hier könnten andere Fonts definiert werden
#ifdef LUCIDA										// Lucida ist ein Proportionalfont
	// deutsche Umlaute siehe "https://www.ascii-code.com/de", es muss der ".._f" Font sein (full)
	#define FONT1 u8g2_font_lubB08_tf				// 4 verschiedene Schriftgrößen
	#define FONT2 u8g2_font_lubB10_tf
	#define FONT3 u8g2_font_lubB12_tf
	#define FONT4 u8g2_font_lubB14_tf
#endif

	myDisplay.setFont(FONT1);						// Font und Schreibfarbe (weiß/hell) für OLED definieren
	myDisplay.setDrawColor(1);
	myDisplay.clearDisplay();						// erst mal alles löschen

	// Zählername auf OLED ausgeben
	printName();

	// Starte WiFi für die Anzeige auf der WEB GUI
	WiFi.persistent(false);							// Auskommentieren wenn Netzwerkname und Passwort in den Flash geschrieben werden sollen.
	WiFi.mode(WIFI_STA);
	WiFi.begin(OwnSSID, OwnPassword);				// -->> die eigenen Credentials oben als Define eintragen <<--

	while (WiFi.status() != WL_CONNECTED) 
	{
		delay(500);
		yield();									// für ESP - ruft WiFi Routinen auf. Wird bereits in delay() aufgerufen.
#ifndef ARDUINO_ESP8266_GENERIC						// ESP-01 - nur beim Wemos Serial. verwenden
		Serial.print(".");							// Debug Ausgabe
#endif
	}

	// Status Wifi auf OLED ausgeben
	printWiFi();
	delay(1000);									// 1 s die Werte (URL usw.) anzeigen

	// Starte Server für die Anzeige auf der WEB GUI
	server.begin();
	printServer();

	server.on("/", handleRoot);
	//server.on("/favicon.ico", handleIcon);		// nicht benötigt (kein Icon festgelegt)
	server.on("/status", handleStatus);
	delay(1000);									// 1 s die Werte (URL usw.) anzeigen

	// IP addresse auf OLED ausgeben
	printIP();
	// \bring up WiFi Ende
	delay(1000);									// 1 s die Werte (URL usw.) anzeigen

	// Starte FileSystem
	LittleFS.begin();
	printFS();
	delay(1000);									// 1 s die Werte (URL usw.) anzeigen

	// Start OTA Server für Update der Firmware über Wifi
	ArduinoOTA.begin();
	ArduinoOTA.onStart([]() {                       // Zählerstand speichern bevor OTA Update durchgeführt wird
		save();
	});

	// bisherige Auflistung 1 Zeile nach oben "scrollen" und OTA ausgeben
	YOFF -= 11;										// so eine Art Scrollfunktion imitieren

	myDisplay.setDrawColor(0);						// untere 4 Zeilen löschen
	myDisplay.drawBox(0, 20, 127, 44);
	myDisplay.setDrawColor(1);

	//printName();									// nicht notwendig, da nur die Liste gelöscht wird
	printServer();
	printIP();
	printFS();
	printOTA();

	File f = LittleFS.open("/value.txt", "r");      // Zählerstand beim Neustart aus Datei lesen
	String str = f.readStringUntil('\n');
	counterValue = str.toInt();
	oldCounterValue = counterValue;
	f.close();
#ifndef ARDUINO_ESP8266_GENERIC						// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.printf("Ulong vom Flash: %li\n", counterValue);
#endif
	delay(2000);									// 2 s die Werte (URL usw.) anzeigen

	DisplayTimer = millis();						// direkt nach PowerOn Display erst mal für DISPLAYPERIOD aktiviert lassen...
}	// \setup()


// ---------------------------------------------------------------------------------------------------------------------
void loop()
{
	// Taster abfragen. Wenn gedrückt, Display für DISPLAYPERIOD einschalten
	if ((digitalRead(TASTER) == LOW) /*&& (bTasterTrig == false) && (bTasterLang == false)*/)
	{
		bTasterTrig = true;							// Taster wurde gedrückt
		DisplayTimer = millis();		// neue Zeit merken
		DisplayContentEin = true;
#ifndef ARDUINO_ESP8266_GENERIC						// ESP-01 - nur beim Wemos Serial. verwenden
				Serial.println("Tastendruck erkannt");
#endif
		if ((bTasterLang == false) && (bTasterTrig == true))
		{
			uTasterLang = millis();
			bTasterLang = true;
#ifndef ARDUINO_ESP8266_GENERIC						// ESP-01 - nur beim Wemos Serial. verwenden
				Serial.println("Lang gestartet");
#endif
		}
		if ((millis() > (uTasterLang + LONGKEY)) && (bTasterLang == true) && (bsaved == false))
		{
#ifndef ARDUINO_ESP8266_GENERIC						// ESP-01 - nur beim Wemos Serial. verwenden
			Serial.println("Langer Tastendruck erkannt");
#endif
			save();									// Zählerwert ins Flash schreiben
			bsaved = true;							// Wert ist gespeichert

			myDisplay.setFont(FONT2);
			myDisplay.drawStr(20, 28, "gespeichert");	// Hinweis für Nutzer
		}
	}
	else	// Display nach Ablauf von DISPLAYPERIOD "abschalten" um Einbrennen zu minimieren
	{		// Würde man normalerweise mit setPowerSave(0) erledigen, das funktioniert bei meinen Displays aber nicht :(
		if ((millis() - DisplayTimer) > DISPLAYPERIOD)
		{
			myDisplay.clearDisplay();
			DisplayContentEin = false;
		}

		bTasterTrig = false;						// Taster wurde zwischenzeitlich losgelassen
		bTasterLang = false;
		bsaved = false;								// Sichern des Zählerwerts wieder scharf schalten
	}

	// Sensor abfragen. Wenn getriggert, Zähler inkrementieren, SensorTLE.triggered wird im IRQ gesetzt
	if (SensorTLE.triggered == true)
	{
#if defined WASSER
		counterValue += 1;									// in der Anzeige der Wasseruhr werden 4 Nachkommastellen (5 mit Metallplatte,
#else														//  6 mit Zahnrad) angezeigt, aber mit dem Magnet wird die 3. Nachkommastelle (Liter) hochgezählt
		counterValue += 10;									// in der Anzeige des Gaszählers werden 3 Nachkommastellen angezeigt, 
#endif														//	aber mit dem Magnet wird die 2. Nachkommastelle hochgezählt

		//DisplayTimer = millis();							// neue Zeit merken (nur zur Programmentwicklung)
		//DisplayContentEin = true;							// Display bei Sensortrigger nur zur Programmentwicklung aktivieren

		SensorTLE.triggered = false;						// Flag löschen, wird per IRQ erneut gesetzt
	}

#ifndef WASSER												// bei der Wasseruhr muss die 3. Nachkommastelle erhalten bleiben
	counterValue = (counterValue / 10) * 10;				// magic um die 3. Nachkommastelle beim Gaszähler immer auf Null (0) zu setzen
#endif	
	float fCounterVal = (float)counterValue / 1000.0;		// für die formatierte Anzeige in float wandeln
	dtostrf(fCounterVal, 10, 3, buffer);					// Zahl formatieren (12 Stellen, 3 Nachkomma, rechtsbündig)

	// Zählerstand im Flash speichern, wenn er sich geändert hat
	unsigned long aktuelleMillis = millis();
	if (aktuelleMillis - letzterAufruf >= 60000UL * 240 && counterValue != oldCounterValue) {       // alle 4 Stunden speichern
		save();
		oldCounterValue = counterValue;
		letzterAufruf = aktuelleMillis;
	}

// WiFi part of loop ----------------------------------------------------------------------------------------------
	server.handleClient();									// WEB GUI bedienen
	ArduinoOTA.handle();									// Check for OTA calls
// \WiFi part of loop ---------------------------------------------------------------------------------------------

	publishMessage();										// Werte auf OLED anzeigen

	// Aktivitätsanzeige auf dem OLED Display
	if ((millis() - activityTimer) > 300)
	{
		if (DisplayContentEin == true)
		{
			propellerState++;
			propellerState = propellerState % 4;

			myDisplay.setFont(FONT1);
			myDisplay.setCursor(126, 7);
			myDisplay.print(propellerString[propellerState]);	// show text
			myDisplay.updateDisplay();
		}
		activityTimer = millis();
	}
}	// \loop()
// ---------------------------------------------------------------------------------------------------------------------

// --------------------------------------------------------------------------------

// --------------------------------------------------------------------------------
// OLED aktualisieren
void publishMessage()										// Ausgabe der Werte auf dem OLED
{
	if (DisplayContentEin == true)
	{
		// Display einmalig nach Setup löschen
		if (clearOnce == true) { myDisplay.clearDisplay(); clearOnce = false; }

		myDisplay.setFont(FONT3);
		printName();
		myDisplay.setFont(FONT2);
		myDisplay.drawStr(0, 40, "Z\xe4hlerstand [m ]");	// ä = \xe4 \228 -> siehe "https://www.ascii-code.com/de"
		myDisplay.drawStr(116, 40, "\xb3");					// das ³ sitzt zu weit links im Buchstabenfeld, muss explizit platziert werden

		myDisplay.setDrawColor(0);							// Zeile mit Zählerwert löschen (der Font ist transparent -> ansonsten Müll)
		myDisplay.drawBox(0, 44, 127, 17);
		myDisplay.setDrawColor(1);

		float fCounterVal = (float)counterValue / 1000.0;	// für die formatierte Anzeige in float wandeln
		dtostrf(fCounterVal, 10, 3, buffer);				// Zahl formatieren (12 Stellen, 3 Nachkomma, rechtsbündig)

		myDisplay.setFont(FONT4);
		myDisplay.drawStr(0, 60, buffer);					// buffer wird in der loop gesetzt
	}
}	// \OLED aktualisieren

// --------------------------------------------------------------------------------
// aktuellen Zählerstand im Flash speichern
void save() {
	File f = LittleFS.open("/value.txt", "w");
	f.printf("%li\n", counterValue);
	f.close();
#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.printf("save()\n");
#endif
}	// \aktuellen Zählerstand im Flash speichern
// --------------------------------------------------------------------------------

// --------------------------------------------------------------------------------
// Routinen zur Ausgabe von Infos
void printName() {
	myDisplay.setFont(FONT3);
	myDisplay.drawStr(0, 16, COUNTERNAME);
	myDisplay.updateDisplay();
}

void printWiFi() {
	myDisplay.setFont(FONT1);
#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.println("");								// Debug Ausgabe
	Serial.println("WiFi  connected");				// Debug Ausgabe
#endif	
	myDisplay.drawStr(0, YOFF, "WiFi  connected");
	myDisplay.updateDisplay();
}

void printServer() {
	myDisplay.setFont(FONT1);
#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.println("Server started");
#endif	
	myDisplay.drawStr(0, YOFF + 11, "Server  started");
	myDisplay.updateDisplay();
}

void printIP() {
#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.print("Use this URL to connect: ");
	Serial.print("http://");
	Serial.print(WiFi.localIP());
	Serial.println("/");
#endif	
	myDisplay.drawStr(0, YOFF + 22, "IP");
	myDisplay.updateDisplay();
	myDisplay.setCursor(16, YOFF + 22);
	myDisplay.print(WiFi.localIP());
	myDisplay.updateDisplay();
}

void printFS() {
	myDisplay.drawStr(0, YOFF + 33, "LittleFS  l\xe4uft");
	myDisplay.updateDisplay();
}

void printOTA() {
#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
	Serial.println("OTA Server  started");
#endif	
	myDisplay.drawStr(0, YOFF + 44, "OTA     started");
	myDisplay.updateDisplay();
}
// \// Routinen zur Ausgabe von Infos

// Routinen für die WEB Oberfläche - Die Routinen stammen ursprünglich aus der Feder von Jens Fleischer (https://fipsok.de/)
//	und wurden von mir nur angepasst bzw. um von mir nicht benötigte Funktionen erleichtert.
// --------------------------------------------------------------------------------
void handleStatus() {
	if (server.arg(0).length() > 0 && server.arg(0).length() <= 9) 	// Format ist 12345.123, also maximal 9 Stellig		
	{
		String svalue = server.arg(0);
		value = svalue.toFloat();

#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
		Serial.printf("server.arg(0).length(): %li\n", server.arg(0).length());

		svalue.toCharArray(buffer, 20);						// für printf in char Array packen..
		Serial.printf("Wert vom Client: %s\n", buffer);		// ..und ausgeben
		svalue.toCharArray(buffer, 20);
		Serial.printf("Zählerstand vom Client: %s\n", buffer);		// als String ausgeben
		Serial.printf("Zählerstand vom Client: %5.3f\n", value);	// als Float ausgeben
#endif	
	counterValue = value * 1000.0;							// 3 Nachkommastellen vor das Komma holen
#ifndef WASSER												// bei der Wasseruhr muss die 3. Nachkommastelle erhalten bleiben
	counterValue = (counterValue / 10) * 10;				// magic um die letzte Stelle immer auf Null (0) zu setzen
#endif	
	save();													// neu eingebenen Zählerstand ins Flash speichern
	}

	char buf[42];
	char tmpbuf[20];
	float fCounterVal = (float)counterValue / 1000.0;		// für die formatierte Anzeige in float wandeln
	dtostrf(fCounterVal, 12, 3, buffer);					// Zahl formatieren (12 Stellen, 3 Nachkomma, rechtsbündig, keine führenden Nullen!)

	String buffi;											// Magic um die Zahl im WEB Interface passend auszurichten (Dezimalpunkt auf der roten Linie)
	for (uint8_t i = 3; i < sizeof(buffer); i++)			// maximal 5 Vorkommastellen, der String hat 12 Stellen insgesamt, also ab 3. Stelle lesen
		if (buffer[i] == ' ') buffi += "0"; 
		else buffi += buffer[i];

	buffi.toCharArray(buffer, sizeof(buffer));

#ifndef ARDUINO_ESP8266_GENERIC								// ESP-01 - nur beim Wemos Serial. verwenden
		Serial.printf("newstring: %ls\n", buffer);
#endif

		snprintf(buf, sizeof(buf), "[\"%s m³\"]", buffer);
	server.send(200, "application/json", buf);				// Zählerstand an Html Seite senden
}	// \handleStatus()


// --------------------------------------------------------------------------------
// Hier habe ich den "Fließtext" von Jens etwas entzerrt um den Überblick für meine Änderungen zu bekommen:
//	-> Keine LEDs auf dem benutzten ESP-01, keine Verwendung von Thingspeak, Zählereingabe 9-stellig (mit Punkt)
// Leider wird das schön formatierte HTML wieder linksbündig in die temp Variable gestopft, aber was solls...

// Html Webseite, der Zählerstand wird alle 5 Sekunden mittels fetch.api aktualisiert
void handleRoot() { 
	String temp = "	<!DOCTYPE HTML>";
	temp += "<html lang='de'>";
	temp += "<head>";
	temp += "<meta charset='UTF-8'>";
	temp += "<meta name='viewport' content='width=device-width, initial-scale=1'>";
#if defined WASSER
	temp += "<title>Wasserzähler</title>";
#else
	temp += "<title>Gaszähler</title>";
#endif
	temp += "<script>";
	temp += "addEventListener('load', () => {";
	temp += "let i = document.querySelector('input'),";
	temp += "n = document.querySelector('#note');";
	temp += "document.querySelector('#grey').addEventListener('click', () => t());";
	temp += "i.addEventListener('blur',";
	temp += "() => {";
	temp += "!i.checkValidity() ? (n.innerHTML = i.validationMessage, n.classList.add('note')) : s('status', i.value)";
	temp += "});";
	temp += "";
	temp += "function t() {";
	temp += "let x = document.getElementById('no');";
	temp += "getComputedStyle(x).display === 'none' ? x.style.display = 'flex' : x.style.display = 'none';";
	temp += "n.innerHTML = '';";
	temp += "n.classList.remove('note');";
	temp += "i.value = ''";
	temp += "}";
	temp += "async function s(p = 'status', v) {";
	temp += "if (Number.isInteger(parseInt(v))) t();";
	temp += "let r = await fetch(p, {";
	temp += "method: 'POST',";
	temp += "body: v";
	temp += "});";
	temp += "let j = await r.json();";
	temp += "document.querySelector('span').innerHTML = j[0];";
	temp += "}";
	temp += "s(), setInterval(s, 5000)";
	temp += "});";
	temp += "</script>";
	temp += "<style>";
	temp += "body {";
	temp += "background-color: tan;";
	temp += "font-size: 1.1em";
	temp += "}";
	temp += "";
	temp += "body,";
	temp += "#grey,";
	temp += "#no,";
	temp += "#black {";
	temp += "display: flex;";
	temp += "flex-flow: column;";
	temp += "align-items: center";
	temp += "}";
	temp += "";
	temp += "#grey {";
	temp += "background-color: #c5c3c3;";
	temp += "width: 340px;";
	temp += "height: 150px;";
	temp += "border: .3em solid #aeaeab;";
	temp += "border-radius: .5em;";
	temp += "box-shadow: 5px 10px 5px #4e4d4d";
	temp += "}";
	temp += "";
	temp += "#no {";
	temp += "background-color: #c5c3c3;";
	temp += "width: 340px;";
	temp += "height: 65px;";
	temp += "border: .3em solid #aeaeab;";
	temp += "border-radius: .5em;";
	temp += "box-shadow: 5px 10px 5px #4e4d4d";
	temp += "}";
	temp += "";
	temp += "#grey {";
	temp += "cursor: pointer";
	temp += "}";
	temp += "";
	temp += "#black {";
	temp += "background-color: black;";
	temp += "width: 14em;";
	temp += "border-radius: .2em";
	temp += "}";
	temp += "";
	temp += "#red {";
	temp += "border: .28em solid red;";
	temp += "position: relative;";
	temp += "width: 3.8em;";
	temp += "height: 1.5em;";
	temp += "left: 1.5em";
	temp += "}";
	temp += "";
	temp += "span {";
	temp += "color: white;";
	temp += "position: relative;";
	temp += "left: .1em;";
	temp += "letter-spacing: .2em;";
	temp += "font-size: 1.6em;";
	temp += "font-weight: bold;";
	temp += "top: 4.55em;";
	temp += "pointer-events: none;";
	temp += "z-index: 1";
	temp += "}";
	temp += "";
	temp += "#no {";
	temp += "display: none";
	temp += "}";
	temp += "";
	temp += ".note {";
	temp += "background-color: #fecdee;";
	temp += "padding:";
	temp += ".3em;";
	temp += "border-radius: .5em;";
	temp += "position: relative;";
	temp += "top: -5.5em";
	temp += "}";
	temp += "";
	temp += "input,";
	temp += "#err {";
	temp += "height: 30px;";
	temp += "margin-top: 1em";
	temp += "}";
	temp += "";
	temp += ".button {";
	temp += "background-color: #75a27e;";
	temp += "height: 40px;";
	temp += "width: 120px;";
	temp += "font-size: 16px;";
	temp += "box-shadow: 5px 5px 5px #4e4d4d";
	temp += "}";
	temp += "";
	temp += ".off {";
	temp += "background-color: #cf848c";
	temp += "}";
	temp += "</style>";
	temp += "</head>";
	temp += "<body>";
	temp += "<span></span>";
	temp += "<div id='grey'>";
	temp += "<h1>Zählerstand ";
#if defined WASSER
	temp += "Wasser";
#else
	temp += "Gas";
#endif
	temp += "</h1>";
		temp += "<div id='black'>";
	temp += "<div id='red'></div>";
	temp += "</div>";
	temp += "</div>";
	temp += "<div id='no'>";
	temp += "<input placeholder=' Zählerstand eingeben' pattern='[0-9]{1,5}[.]\\d{0,3}' title='Nur Ziffern, 1..5 vor dem Punkt und 0..3 nach dem Punkt sowie Punkt sind erlaubt' required>";
	temp += "<div id='note'></div>";
	temp += "</body>";
	temp += "</html>";

server.send(200, "text/html", temp);
}    // \handleRoot()
